@@ -25,10 +25,10 @@ from qwidget_gui import __version__
 
 import dbus
 
-# from dbus.mainloop.glib import DBusGMainLoop
+import dbus.mainloop.glib
 
-from dbus.mainloop.qt import DBusQtMainLoop
-DBusQtMainLoop(set_as_default=True)
+#from dbus.mainloop.qt import DBusQtMainLoop
+#DBusQtMainLoop(set_as_default=True)
 
 import twitter
 import glob
@@ -70,7 +70,7 @@ def install_excepthook(version):
     sys.excepthook = my_excepthook
 
 
-class Daemon:
+class Daemon(object):
 
     """
     A generic daemon class.
@@ -173,7 +173,6 @@ class Daemon:
                 sys.stderr.write(message % self.pidfile)
                 sys.exit(1)
             except OSError:
-
                 sys.stderr.write('pidfile %s already exist. But daemon is dead.\n'
                                   % self.pidfile)
 
@@ -182,6 +181,9 @@ class Daemon:
         self.daemonize()
         self.run()
 
+    def debug(self):
+        self.run()
+        
     def stop(self):
         """
         Stop the daemon
@@ -259,7 +261,9 @@ class KhweeteurDBusHandler(dbus.service.Object):
             m_notify = m_bus.get_object('org.freedesktop.Notifications',
                                         '/org/freedesktop/Notifications')
             iface = dbus.Interface(m_notify, 'org.freedesktop.Notifications')
-            iface.SystemNoteInfoprint('Khweeteur : ' + message)
+            if not message.startswith('Khweeteur'):
+                message = 'Khweeteur : ' + message
+            iface.SystemNoteInfoprint(message)
         except:
             import traceback
             (exc_type, exc_value, exc_traceback) = sys.exc_info()
@@ -267,6 +271,11 @@ class KhweeteurDBusHandler(dbus.service.Object):
                           % repr(traceback.format_exception(exc_type,
                           exc_value, exc_traceback)))
 
+    @dbus.service.method(dbus_interface='com.example.Sample',
+                         in_signature='', out_signature='b')
+    def isRunning(self):
+        return True
+        
     @dbus.service.signal(dbus_interface='net.khertan.Khweeteur', signature='')
     def refresh_ended(self):
         pass
@@ -275,13 +284,19 @@ class KhweeteurDBusHandler(dbus.service.Object):
     def new_tweets(self, count, ttype):
         logging.debug('New tweet notification ttype : %s (%s)' % (ttype,
                       str(type(ttype))))
-        if ttype in ('Mentions', 'DMs'):
+        settings = QSettings('Khertan Software', 'Khweeteur')
+        #.value('showNotifications') == '2':                      
+
+        if ((ttype == 'Mentions') and
+            (settings.value('showDMNotifications') == '2')) \
+            or ((ttype == 'DMs') and
+                (settings.value('showDMNotifications') == '2')):
             m_bus = dbus.SystemBus()
             m_notify = m_bus.get_object('org.freedesktop.Notifications',
                                         '/org/freedesktop/Notifications')
             iface = dbus.Interface(m_notify, 'org.freedesktop.Notifications')
 
-            if ttype == 'DMs':
+            if ttype == 'DMs' :
                 msg = 'New DMs'
             elif ttype == 'Mentions':
                 msg = 'New mentions'
@@ -310,9 +325,18 @@ class KhweeteurDBusHandler(dbus.service.Object):
                 pass
 
 
-class KhweeteurDaemon(Daemon):
+class KhweeteurDaemon(Daemon,QCoreApplication):
 
+    def __init__(self, pidfile,
+        stdin='/dev/null',
+        stdout='/dev/null',
+        stderr='/dev/null',):
+        Daemon.__init__(self,pidfile,stdin=stdin, \
+                        stdout=stdout, \
+                        stderr=stderr)
     def run(self):
+        QCoreApplication.__init__(self,sys.argv)
+        dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
         try:
             from PySide import __version_info__ as __pyside_version__
         except:
@@ -322,7 +346,7 @@ class KhweeteurDaemon(Daemon):
         except:
             __qt_version__ = None
 
-        app = QCoreApplication(sys.argv)
+#        app = QCoreApplication(sys.argv)
 
         logging.basicConfig(level=logging.DEBUG,
                             format='%(asctime)s %(levelname)-8s %(message)s',
@@ -331,6 +355,12 @@ class KhweeteurDaemon(Daemon):
         logging.info('Starting daemon %s' % __version__)
         logging.info('PySide version %s' % repr(__pyside_version__))
         logging.info('Qt Version %s' % repr(__qt_version__))
+
+        try:
+            os.nice(10)
+        except:
+            pass
+            
         self.bus = dbus.SessionBus()
         self.bus.add_signal_receiver(self.update, path='/net/khertan/Khweeteur'
                                      , dbus_interface='net.khertan.Khweeteur',
@@ -363,7 +393,7 @@ class KhweeteurDaemon(Daemon):
             os.makedirs(self.post_path)
 
         self.dbus_handler = KhweeteurDBusHandler()
-
+                
         # mainloop = DBusQtMainLoop(set_as_default=True)
 
         settings = QSettings('Khertan Software', 'Khweeteur')
@@ -371,19 +401,17 @@ class KhweeteurDaemon(Daemon):
             refresh_interval = 600
         else:
             refresh_interval = int(settings.value('refresh_interval')) * 60
-            if refresh_interval < 600:
-                refresh_interval = 600
 
         self.utimer = QTimer()
         self.utimer.timeout.connect(self.update)
         self.utimer.start(refresh_interval * 1000)
-
+                
         QTimer.singleShot(200, self.update)
 
         # gobject.timeout_add_seconds(refresh_interval, self.update, priority=gobject.PRIORITY_LOW)
 
         logging.debug('Timer added')
-        app.exec_()
+        self.exec_()
         logging.debug('Daemon stop')
 
     def post_tweet(
@@ -560,7 +588,7 @@ class KhweeteurDaemon(Daemon):
                                 if settings.contains('ShowInfos'):
                                     if settings.value('ShowInfos') == '2':
                                         self.dbus_handler.info('Khweeteur: Retweet posted to '
-         + account['name'])
+                                             + account['name'])
                         elif post['action'] == 'tweet':
 
                             # Else "simple" tweet
@@ -697,6 +725,7 @@ class KhweeteurDaemon(Daemon):
                           exc_value, exc_traceback)))
 
     def retrieve(self, options=None):
+        logging.debug('Start update')
         settings = QSettings('Khertan Software', 'Khweeteur')
 
         showInfos = False
@@ -718,11 +747,14 @@ class KhweeteurDaemon(Daemon):
             logging.debug('Setting synced')
 
             # Cleaning old thread reference for keep for gc
+            logging.debug('Number of thread not gc : %s' % str(len(self.threads)))
 
-            for thread in self.threads:
-                if not thread.isAlive():
-                    self.threads.remove(thread)
-                    logging.debug('Removed a thread')
+            if len(self.threads)>0:
+                return            
+#            for thread in self.threads:
+#                if not thread.isAlive():
+#                    self.threads.remove(thread)
+#                    logging.debug('Removed a thread')
 
             # Remove old tweets in cache according to history prefs
 
@@ -775,7 +807,7 @@ class KhweeteurDaemon(Daemon):
             for index in range(nb_accounts):
                 settings.setArrayIndex(index)
                 access_token = settings.value('access_token')
-                if not access_token in self.apis:
+                if not access_token in self.apis:                    
                     self.apis[access_token] = self.get_api(dict((key,
                             settings.value(key)) for key in settings.allKeys()))
                     self.me_users[access_token] = None
@@ -787,6 +819,8 @@ class KhweeteurDaemon(Daemon):
                     except Exception, err:
                         self.me_users[access_token] = None
                         logging.error('VerifyCredential2 : %s' % str(err))
+                        self.dbus_handler.refresh_ended()
+                        logging.debug('Finished loop')
                         if showInfos:
                             self.dbus_handler.info(str(err))
 
@@ -800,19 +834,25 @@ class KhweeteurDaemon(Daemon):
                     # Worker
                     try:
                         self.threads.append(KhweeteurRefreshWorker(api,
-                                'HomeTimeline', self.dbus_handler, me_user_id))
+                                'HomeTimeline', me_user_id))
+                        self.threads[-1].error.connect(self.dbus_handler.info)                        
+                        self.threads[-1].new_tweets.connect(self.dbus_handler.new_tweets) 
                     except Exception, err:
                         logging.error('Timeline : %s' % str(err))
 
                     try:
                         self.threads.append(KhweeteurRefreshWorker(api,
-                                'Mentions', self.dbus_handler, me_user_id))
+                                'Mentions', me_user_id))
+                        self.threads[-1].error.connect(self.dbus_handler.info)                        
+                        self.threads[-1].new_tweets.connect(self.dbus_handler.new_tweets) 
                     except Exception, err:
                         logging.error('Mentions : %s' % str(err))
 
                     try:
                         self.threads.append(KhweeteurRefreshWorker(api, 'DMs',
-                                self.dbus_handler, me_user_id))
+                                 me_user_id))
+                        self.threads[-1].error.connect(self.dbus_handler.info)                        
+                        self.threads[-1].new_tweets.connect(self.dbus_handler.new_tweets) 
                     except Exception, err:
                         logging.error('DMs : %s' % str(err))
 
@@ -820,15 +860,19 @@ class KhweeteurDaemon(Daemon):
                     for terms in searches:
                         try:
                             self.threads.append(KhweeteurRefreshWorker(api,
-                                    'Search:' + terms, self.dbus_handler,
+                                    'Search:' + terms,
                                     me_user_id))
+                            self.threads[-1].error.connect(self.dbus_handler.info)                        
+                            self.threads[-1].new_tweets.connect(self.dbus_handler.new_tweets)                                     
                         except Exception, err:
                             logging.error('Search %s: %s' % (terms, str(err)))
 
                     # Start retrieving the list
                     try:
                         self.threads.append(KhweeteurRefreshWorker(api,
-                                'RetrieveLists', self.dbus_handler, me_user_id))
+                                'RetrieveLists', me_user_id))
+                        self.threads[-1].error.connect(self.dbus_handler.info)                        
+                        self.threads[-1].new_tweets.connect(self.dbus_handler.new_tweets) 
                     except Exception, err:
                         logging.error('Retrieving List error %s' % (str(err), ))
 
@@ -842,7 +886,9 @@ class KhweeteurDaemon(Daemon):
                                         'Near:%s:%s' % \
                                            (str(self.geoloc_coordinates[0]),
                                             str(self.geoloc_coordinates[1])),
-                                        self.dbus_handler, me_user_id))
+                                        me_user_id))
+                                self.threads[-1].error.connect(self.dbus_handler.info)                        
+                                self.threads[-1].new_tweets.connect(self.dbus_handler.new_tweets) 
                             except Exception, err:
                                 logging.error('Near: %s' % (str(err)))
 
@@ -852,7 +898,9 @@ class KhweeteurDaemon(Daemon):
                         try:
                             self.threads.append(KhweeteurRefreshWorker(api,
                                     'List:' + user + ':' + list_id,
-                                    self.dbus_handler, me_user_id))
+                                    me_user_id))
+                            self.threads[-1].error.connect(self.dbus_handler.info)                        
+                            self.threads[-1].new_tweets.connect(self.dbus_handler.new_tweets) 
                         except Exception, err:
                             logging.error('List %s: %s' % (list_id, str(err)))
 
@@ -862,26 +910,36 @@ class KhweeteurDaemon(Daemon):
                             logging.debug('Try to run Thread : %s'
                                     % str(thread))
                             try:
+                                self.threads[idx].finished.connect(self.athread_end)
+                                self.threads[idx].terminated.connect(self.athread_end)
                                 self.threads[idx].start()
+                                
                             except RuntimeError, err:
                                 logging.debug('Attempt to start a thread already running : %s'
                                          % (str(err), ))
-                    except:
-                        logging.error('Running Thread error')
+                    except Exception, err:
+                        logging.error('Running Thread error: %s' % err)
 
             settings.endArray()
 
-            while any([thread.isAlive() for thread in self.threads]):
-                time.sleep(2)
-
-            self.dbus_handler.refresh_ended()
-
-            logging.debug('Finished loop')
         except Exception, err:
-
             logging.exception(str(err))
-            logging.debug(str(err))
 
+    @Slot()
+    def kill_thread(self):
+        for thread in self.threads:
+            if not thread.isFinished():
+                thread.terminate()             
+    @Slot()
+    def athread_end(self):
+        try:
+            self.threads.remove(self.sender())
+        except ValueError:
+            pass
+        if all([thread.isFinished() for thread in self.threads]):            
+            self.dbus_handler.refresh_ended()
+            logging.debug('Finished loop')
+        
 
 if __name__ == '__main__':
     install_excepthook(__version__)
@@ -898,11 +956,13 @@ if __name__ == '__main__':
             daemon.stop()
         elif 'restart' == sys.argv[1]:
             daemon.restart()
+        elif 'debug' == sys.argv[1]:
+            daemon.debug()
         else:
             logging.error('Unknown command')
             print 'Unknown command'
             sys.exit(2)
         sys.exit(0)
     else:
-        print 'usage: %s start|stop|restart|startfromprefs' % sys.argv[0]
+        print 'usage: %s start|stop|restart|startfromprefs|debug' % sys.argv[0]
         sys.exit(2)
